/**
 * © 2008 by David ‘Bombe’ Roden <bombe@pterodactylus.net>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ED2KLink.h"
#include "GrowingBuffer.h"
#include "GlobalSettings.h"

static int getDigits(size_t number) {
	int digits = 0;
	do {
		number /= 10;
		digits++;
	} while (number);
	return digits;
}

ED2KLink::ED2KLink(const char* filename, const size_t size, const void* hash) {
	this->filename = (char*) malloc(strlen(filename));
	this->hash = malloc(16);

	strcpy(this->filename, filename);
	memcpy(this->hash, hash, 16);
	this->size = size;

	link = (char*) calloc(13 + (strlen(filename) + 1) + 1 + getDigits(size) + 1 + 32 + 2, 1);
	sprintf(link, "ed2k://|file|%s|%u|", filename, size);
	for (int i = 0; i < 16; i++) {
		sprintf(link + strlen(link), "%02x", *((int*) this->hash + i));
	}
	sprintf(link + strlen(link), "|/");
	GlobalSettings::isVerbose() && fprintf(stderr, "[%s:%d] created ed2kLink: %s\n", __FILE__, __LINE__, link);
}

ED2KLink::~ED2KLink() {
	if (link) {
		free(link);
	}
	if (filename) {
		free(filename);
	}
	if (hash) {
		free(hash);
	}
}

ED2KLink* ED2KLink::parseED2KLink(const char* buffer) {
	const char* tempBuffer = buffer;
	char* filename;
	char* sizeString;
	size_t size;
	int hashIndex;
	char hash[16];
	GrowingBuffer growingBuffer;

	GlobalSettings::isVerbose() && fprintf(stderr, "[%s:%d] trying to parse “%s”...\n", __FILE__, __LINE__, buffer);

	if (strncmp("ed2k://|file|", buffer, 13)) {
		return NULL;
	}

	tempBuffer += 13;
	while (*tempBuffer != '|') {
		growingBuffer.write(tempBuffer++, 1);
	}
	filename = (char*) malloc(growingBuffer.getLimit() + 1);
	growingBuffer.read(filename, growingBuffer.getLimit());
	growingBuffer.clear();
	tempBuffer++;

	GlobalSettings::isVerbose() && fprintf(stderr, "[%s:%d] parsed filename: “%s”.\n", __FILE__, __LINE__, filename);

	while (*tempBuffer != '|') {
		growingBuffer.write(tempBuffer++, 1);
	}
	sizeString = (char*) malloc(growingBuffer.getLimit() + 1);
	growingBuffer.read(sizeString, growingBuffer.getLimit());
	growingBuffer.clear();
	tempBuffer++;

	size = atol(sizeString);
	GlobalSettings::isVerbose() && fprintf(stderr, "[%s:%d] parsed size “%s” into %d.\n", __FILE__, __LINE__, sizeString, size);

	for (hashIndex = 0; hashIndex < 16; hashIndex++, tempBuffer += 2) {
		sscanf(tempBuffer, "%hhx", hash + hashIndex);
	}

	return new ED2KLink(filename, size, hash);
}

const char* ED2KLink::getLink() {
	return this->link;
}

const char* ED2KLink::getFilename() {
	return this->filename;
}

const size_t ED2KLink::getSize() {
	return this->size;
}

const void* ED2KLink::getHash() {
	return this->hash;
}


